
// -----------------------------
// -------- C++ Includes -------
#include <stdio.h>
#include <iostream>
#include <string>

// -------------------------------
// ------- Matlab Includes -------
#include <matrix.h>
#include <mex.h>

/**************************************************
 *@brief Main Matlab Function
 *@param nlhs - Amount of LeftHandSide Parameters
 *@param nrhs - Amount of RightHandside Parameters
 *@param plhs - Array of Output Arguments
 *@param prhs - Array of Input Arguments
 ***************************************************/
void mexFunction(int nlhs, mxArray* plhs[], int nrhs, const mxArray* prhs[]) {

	/*******************************
	 *@brief: Pointer to First Array 
	 ******************************/
	const mxArray* Matlab_Input = prhs[0];

	/**************************************
	 *@brief: Dimensions of the First Array 
	 **************************************/
	const mwSize* Dimensions_First = mxGetDimensions(Matlab_Input);
	size_t Nx = static_cast<size_t>(Dimensions_First[1]);
	size_t Ny = static_cast<size_t>(Dimensions_First[0]);

	/*******************************************
	 *@brief: Create output Array with same size
	 * Could as well be "mxDuplicateArray()"
	 *******************************************/
	const size_t OutputSize[] = { static_cast<size_t>(Ny) ,static_cast<size_t>(Nx) };
	plhs[0] = mxCreateNumericArray(2, OutputSize, mxDOUBLE_CLASS, mxREAL);

	/**********************
	 *@brief: Grab Pointers  
	 **********************/
	mxDouble* I_Data = mxGetDoubles(prhs[0]);
	mxDouble* O_Data = mxGetDoubles(plhs[0]);
	
	/************************************
	 *@brief: Do some Magic with Data ... 
	 ************************************/
	for (size_t i = 0; i < Ny; i++) {
		for (size_t j = 0; j < Nx; j++) {
			size_t Index = j + i * Nx;
			O_Data[Index] = I_Data[Index] + 1.0;
		}
	}

	return;
}